import threading
import time
import unittest
from unittest.mock import Mock, patch

import requests

from configalchemy.contrib.apollo import ApolloBaseConfig, ConfigException, logger


class ApolloConfigTestCase(unittest.TestCase):
    @patch.object(requests, "get")
    @patch.object(ApolloBaseConfig, "start_long_poll")
    def test_ApolloConfig(self, start_long_poll, requests_get):
        return_value = {
            "namespaceName": "application",
            "configurations": {"TEST": "test"},
        }

        requests_get.side_effect = lambda *a, **kw: Mock(
            status_code=200, ok=True, json=Mock(return_value=return_value)
        )

        class DefaultConfig(ApolloBaseConfig):
            TEST = "base"
            #: apollo
            APOLLO_SERVER_URL = ""
            APOLLO_APP_ID = ""
            APOLLO_CLUSTER = "default"
            APOLLO_NAMESPACE = "application"

        config = DefaultConfig()
        config.start_long_poll()
        self.assertEqual("test", config["TEST"])
        self.assertEqual(1, start_long_poll.call_count)

    @patch.object(requests, "get")
    @patch.object(logger, "debug")
    @patch.object(time, "sleep")
    def test_long_poll(self, time_sleep, logging_debug, requests_get):
        class DefaultConfig(ApolloBaseConfig):
            TEST = "base"
            CONFIGALCHEMY_ENABLE_FUNCTION = True
            #: apollo
            APOLLO_SERVER_URL = ""
            APOLLO_APP_ID = ""
            APOLLO_CLUSTER = "default"
            APOLLO_NAMESPACE = "application"
            APOLLO_EXTRA_NAMESPACE = "test"

        application_return_value = {
            "namespaceName": "application",
            "configurations": {"TEST": "application"},
        }

        def mock_get(url: str, **kwargs):
            if "notifications" in url:
                return Mock(
                    status_code=200,
                    ok=False,
                    json=Mock(
                        return_value=[
                            {"namespaceName": "application", "notificationId": 1}
                        ]
                    ),
                )
            elif "test" in url:
                return Mock(
                    status_code=200,
                    ok=True,
                    json=Mock(
                        return_value={
                            "namespaceName": "test",
                            "configurations": {"TEST": "test"},
                        }
                    ),
                )
            else:
                return Mock(
                    status_code=200,
                    ok=True,
                    json=Mock(return_value=application_return_value),
                )

        requests_get.side_effect = mock_get
        config = DefaultConfig()
        self.assertEqual("application", config.TEST)
        application_return_value["configurations"] = {"TEST": "changed"}

        count = 0

        def logging_debug_call(*args, **kwargs):
            nonlocal count
            if count == 0:
                count += 1
                return 0
            elif count == 1:
                count += 1
                return 99999
            else:
                raise ConfigException("break")

        logging_debug.side_effect = logging_debug_call
        time_sleep.side_effect = ConfigException("break")

        with self.assertRaises(ConfigException):
            config.long_poll()

        self.assertEqual("changed", config.TEST)
        self.assertIn("application", config.apollo_notification_map)
        self.assertIn("test", config.apollo_notification_map)

    @patch.object(threading, "Thread")
    def test_start_long_poll(self, thread_mock):
        class DefaultConfig(ApolloBaseConfig):
            CONFIGALCHEMY_ENABLE_FUNCTION = False

        config = DefaultConfig()
        config.start_long_poll()
        thread_mock.assert_called_with(target=config.long_poll)


if __name__ == "__main__":
    unittest.main()



import json
import unittest
from unittest.mock import Mock

from configalchemy.meta import ConfigMeta, ConfigMetaJSONEncoder


class MetaTestCase(unittest.TestCase):
    def test_usage(self):
        default_value = 0
        int_field = Mock()
        config_meta = ConfigMeta(default_value=default_value, field=int_field)
        with config_meta:
            priority = 0
            int_field.validate = Mock(return_value=priority)
            config_meta.set(priority, 0)
            self.assertEqual(0, config_meta.value)
            priority = 10
            int_field.validate = Mock(return_value=priority)
            config_meta.set(priority, 0)
            self.assertEqual(priority, config_meta.value)
            priority = 5
            int_field.validate = Mock(return_value=priority)
            config_meta.set(priority, 0)
            self.assertEqual(10, config_meta.value)

            self.assertEqual(0, config_meta.items[0].priority)
            self.assertEqual(0, config_meta.items[0].value)
            self.assertEqual(0, config_meta.items[1].priority)
            self.assertEqual(0, config_meta.items[1].value)
            self.assertEqual(5, config_meta.items[2].priority)
            self.assertEqual(5, config_meta.items[2].value)
            self.assertEqual(10, config_meta.items[3].priority)
            self.assertEqual(10, config_meta.items[3].value)

    def test_json_encode(self):
        default_value = 0
        int_field = Mock()
        config_meta = ConfigMeta(default_value=default_value, field=int_field)
        self.assertEqual(
            json.dumps({"TEST": default_value}),
            json.dumps({"TEST": config_meta}, cls=ConfigMetaJSONEncoder),
        )
        self.assertEqual("0", str(config_meta))
        self.assertEqual("0", repr(config_meta))
        self.assertEqual(
            "[ConfigMetaItem(priority=0, value=0)]", str(config_meta.items)
        )


if __name__ == "__main__":
    unittest.main()



"""Tests for `configalchemy` package."""
import json
import os
import unittest
import asyncio
from configalchemy import BaseConfig, ConfigType


class ConfigalchemyTestCase(unittest.TestCase):
    """Tests for `configalchemy` package."""

    json_file = "test.json"

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    @classmethod
    def setUpClass(cls) -> None:
        with open(cls.json_file, "w") as fp:
            json.dump({"JSON_TEST": "JSON_TEST"}, fp)

    @classmethod
    def tearDownClass(cls) -> None:
        os.remove(cls.json_file)

    def test_default_config_init_from_object(self):
        class DefaultConfig(BaseConfig):
            TEST = "test"

        config = DefaultConfig()

        self.assertEqual("test", config["TEST"])
        self.assertEqual("test", config.TEST)
        with self.assertRaises(KeyError):
            _ = config["NOT_EXIST"]

        def test_unpack(**kw):
            self.assertEqual(config, kw)

        test_unpack(**config)
        self.assertEqual("NOT_EXIST", config.get("NOT_EXIST", "NOT_EXIST"))

        self.assertTrue(config)
        self.assertTrue("TEST" in config)
        self.assertEqual(len(config), len(list(iter(config))))

        config.update(dict(TEST="update"))
        config.update(dict(TEST="updated"))
        self.assertEqual("updated", config["TEST"])
        self.assertEqual("updated", config.TEST)
        del config["TEST"]
        del config["TEST"]
        self.assertEqual("test", config.TEST)
        self.assertEqual("test", DefaultConfig.TEST)
        self.assertEqual("test", config["TEST"])
        self.assertEqual("test", json.loads(config.json())["TEST"])
        self.assertEqual(str(config), str(json.loads(config.json())))

    def test_default_config_update_from_json(self):
        class DefaultConfig(BaseConfig):
            CONFIGALCHEMY_CONFIG_FILE = self.json_file

            JSON_TEST = "default"

        config = DefaultConfig()
        self.assertEqual("JSON_TEST", config["JSON_TEST"])
        self.assertEqual("JSON_TEST", config.JSON_TEST)

    def test_config_file_not_exist(self):
        class DefaultConfig(BaseConfig):
            CONFIGALCHEMY_ENV_PREFIX = "CONFIGALCHEMY_"
            CONFIGALCHEMY_CONFIG_FILE = "not.exist"

        with self.assertRaises(IOError):
            DefaultConfig()

        os.environ.setdefault("CONFIGALCHEMY_CONFIGALCHEMY_LOAD_FILE_SILENT", "True")
        DefaultConfig()

    def test_config_with_env(self):
        os.environ["test_TEST"] = "changed"

        class DefaultConfig(BaseConfig):
            CONFIGALCHEMY_ENV_PREFIX = "test_"
            TEST = "default"

        config = DefaultConfig()
        self.assertEqual("changed", config["TEST"])
        self.assertEqual("changed", config.TEST)

    def test_update_config_from_function(self):
        class DefaultConfig(BaseConfig):
            CONFIGALCHEMY_ENABLE_FUNCTION = True
            TEST = "default"

            def configuration_function(self) -> ConfigType:
                return {"TEST": "changed"}

        config = DefaultConfig()
        self.assertEqual("changed", config["TEST"])
        self.assertEqual("changed", config.TEST)

    def test_lowerest_priority_update(self):
        class DefaultConfig(BaseConfig):
            CONFIGALCHEMY_ENABLE_FUNCTION = True
            CONFIGALCHEMY_FUNCTION_VALUE_PRIORITY = -1
            TEST = "default"

            def configuration_function(self) -> ConfigType:
                return {"TEST": "changed"}

        config = DefaultConfig()
        self.assertEqual("default", config["TEST"])
        self.assertEqual("default", config.TEST)
        self.assertEqual(-1, config.meta["TEST"].items[0].priority)
        self.assertEqual("changed", config.meta["TEST"].items[0].value)

    def test_async_update_config_from_function(self):
        class DefaultConfig(BaseConfig):
            CONFIGALCHEMY_ENABLE_FUNCTION = True
            TEST = "default"

            async def configuration_function(self) -> ConfigType:
                return {"TEST": "changed"}

        config = DefaultConfig()
        self.assertEqual("changed", config["TEST"])
        self.assertEqual("changed", config.TEST)

    def test_async_update_config_from_function_within_event_loop(self):
        class DefaultConfig(BaseConfig):
            CONFIGALCHEMY_ENABLE_FUNCTION = True
            TEST = "default"

            async def configuration_function(self) -> ConfigType:
                return {"TEST": "changed"}

        async def test():
            config = DefaultConfig()
            self.assertEqual("changed", config["TEST"])
            self.assertEqual("changed", config.TEST)

        loop = asyncio.get_event_loop()
        loop.run_until_complete(test())
        loop.close()

    def test_config_priority(self):
        os.environ["test_FOURTH"] = "4"
        current_json_file = "test_priority.json"

        with open(current_json_file, "w") as fp:
            json.dump({"THIRD": "3", "FOURTH": "3"}, fp)

        class DefaultConfig(BaseConfig):
            # env
            CONFIGALCHEMY_ENV_PREFIX = "test_"
            # file
            CONFIGALCHEMY_CONFIG_FILE = current_json_file
            CONFIGALCHEMY_ENABLE_FUNCTION = True

            FIRST = 1
            SECOND = "1"
            THIRD = "1"
            FOURTH = 1

            def configuration_function(self) -> ConfigType:
                return {"SECOND": "2", "THIRD": 2, "FOURTH": "2"}

        config = DefaultConfig()
        os.remove(current_json_file)

        self.assertEqual(1, config["FIRST"])
        self.assertEqual("2", config["SECOND"])
        self.assertEqual("3", config["THIRD"])
        self.assertEqual(4, config["FOURTH"])
        config.access_config_from_function(config.CONFIGALCHEMY_FUNCTION_VALUE_PRIORITY)
        self.assertEqual(4, config["FOURTH"])

    def test_multiple_inheritance(self):
        class AConfig(BaseConfig):
            A_TEST = "TEST"

        class BConfig(BaseConfig):
            B_TEST = "TEST"

        class CConfig(AConfig, BConfig):
            CONFIGALCHEMY_ENABLE_FUNCTION = True

            def configuration_function(self) -> ConfigType:
                return {"A_TEST": "A_TEST", "B_TEST": "B_TEST"}

        self.assertEqual("A_TEST", CConfig().A_TEST)
        self.assertEqual("B_TEST", CConfig().B_TEST)

    def test_config_key_without_default_value(self):
        class DefaultConfig(BaseConfig):
            CONFIGALCHEMY_ENABLE_FUNCTION = True
            CONFIGALCHEMY_CONFIG_FILE = self.json_file

            def configuration_function(self) -> ConfigType:
                return {"JSON_TEST": "function"}

        config = DefaultConfig()
        self.assertEqual("JSON_TEST", config.JSON_TEST)
        self.assertEqual(
            config.CONFIGALCHEMY_CONFIG_FILE_VALUE_PRIORITY,
            config.meta["JSON_TEST"].items[1].priority,
        )

    def test_config_with_property(self):
        class DefaultConfig(BaseConfig):
            PREFIX = "prefix"
            NAME = "name"

            @property
            def FULL_NAME(self):
                return f"{self.PREFIX}-{self.NAME}"

        config = DefaultConfig()
        config.NAME = "world"
        self.assertEqual("prefix-world", config.FULL_NAME)



import json
import unittest
from typing import Optional, List, Any
from unittest.mock import Mock

from configalchemy.field import Field, ValidateException
from configalchemy.types import Json


class FieldTestCase(unittest.TestCase):
    def test_validate(self):
        int_field = Field(name="TEST", default_value=0, annotation=None)
        for value in [b"0", "0"]:
            self.assertEqual(0, int_field.validate(value))
        with self.assertRaises(ValidateException) as e:
            int_field.validate(".0")
        self.assertIn(".0", str(e.exception))
        self.assertIn(str(type(".0")), str(e.exception))
        self.assertEqual("TEST", e.exception.name)
        self.assertEqual(".0", e.exception.value)

    def test_bool_validate(self):
        bool_field = Field(name="TEST", default_value=False, annotation=None)
        for value in ["true", "1", "yes", "y", 1]:
            self.assertTrue(bool_field.validate(value))
        for value in ["0", "false", "False", "No", 0]:
            self.assertFalse(bool_field.validate(value))
        self.assertTrue(bool_field.validate(True))
        self.assertFalse(bool_field.validate(False))

    def test_union_type(self):
        optional_field = Field(
            name="TEST", default_value=None, annotation=Optional[int]
        )
        self.assertEqual(1, optional_field.validate(1))
        self.assertEqual(1, optional_field.validate("1"))

    def test_json_type(self):
        value_type = Json[list]
        self.assertIs(value_type, Json[list])
        default_value: value_type = []
        json_field = Field(
            name="TEST", default_value=default_value, annotation=value_type
        )
        self.assertEqual([1], json_field.validate([1]))
        self.assertEqual([1], json_field.validate(json.dumps([1])))

        default_value: Json[List[int]] = [1, 2]
        json_field = Field(
            name="TEST", default_value=default_value, annotation=Json[List[int]]
        )
        self.assertEqual([1], json_field.validate(json.dumps([1])))

    def test_generic_field(unittest_self):
        class MyType:
            ...

        my_type = MyType()
        generic_field = Field(name="TEST", default_value=my_type, annotation=None)
        unittest_self.assertEqual(my_type, generic_field.validate(my_type))
        with unittest_self.assertRaises(ValidateException):
            generic_field.validate("typecast")

        value = ["1", "2"]
        typecast = Mock(return_value=value)

        class TestGenericConfigMixin:
            @classmethod
            def __type_check__(cls, instance) -> bool:
                return isinstance(instance, list)

            @classmethod
            def __typecast__(cls, value: Any) -> list:
                return typecast(value)

        generic_config = TestGenericConfigMixin()
        generic_field = Field(
            name="TEST", default_value=generic_config, annotation=None
        )
        unittest_self.assertEqual(value, generic_field.validate(value))
        unittest_self.assertFalse(typecast.called)
        unittest_self.assertEqual(value, generic_field.validate("typecast"))
        typecast.assert_called_with("typecast")


if __name__ == "__main__":
    unittest.main()
